@@ -3,7 +3,7 @@
 #############################
 
 from .core import *
-from .__array import array
+from .rparray import array
 from .event import *
 from .functionality import *
 
@@ -65,12 +65,13 @@ def map(list1,list2:array=None,name1='',name2='',size:int=500,long=-90,lat=30):
     return fig
 
 # Plot the dictionary data obtained from a buffer:
-def plot_buffer(data,title='Plots of Buffer data'):
+def plot_buffer(data,title='Plots of Buffer data',UNITS=None):
     """Plot buffer automatically
 
     Args:
         data (_type_): Buffer that we want to plot
         title (str, optional): Title of the plot. Defaults to 'Plots of Buffer data'.
+        UNITS (_type_,optional): List of strings to be used as units for the plots.
 
     Returns:
         fig,ax: Matplotlib figure and axis
@@ -86,10 +87,18 @@ def plot_buffer(data,title='Plots of Buffer data'):
     fig.suptitle(title,fontsize=18)
     axes = axes.flatten()
 
+    # Handle units
+    if UNITS is None: UNITS = ['']*len(keys)
+    assert(len(UNITS) == len(keys))
+
     # Plot each of the data points
-    for i,key,ax in zip(range(len(axes)),keys,axes):
+    for i,key,ax,unit in zip(range(len(axes)),keys,axes,UNITS):
         ax.plot(events,data[key],c=colors[i%len(colors)],lw=0.7)
-        ax.set_title(key.title())
+        ax.set_title(key.title()+' '+unit)
+
+        # Adding x tick labels
+        if i != len(axes) -1:
+            ax.xaxis.set_tick_params(which='both',labelbottom=True)
 
         # Customize the plot style
         ax.tick_params(axis='both',which='both',direction='in',top=True,right=True)
@@ -141,9 +150,13 @@ def plot_hists(data,struct=NONVETO_STRUCT,bins=600,RANGE=None):
     ax      = ax.flatten()
     colors  = cm.get_cmap('Dark2').colors
 
+    # Get the maximum energy
+    ADC_max = max([max(channel['adc_counts']) for channel in channels])
+
+
     # Plot the histogram of each channel
     for i,channel in enumerate(channels):
-        ax[i].hist(channel['adc_counts'],bins=bins,range=RANGE,color=colors[i%len(channels)])
+        ax[i].hist(channel['adc_counts'],bins=int(bins/ADC_max*max(channel['adc_counts'])),range=RANGE,color=colors[i%len(channels)])
 
         ax[i].set_title('Energy of Channel: %d'%i)
         ax[i].set_yscale('log')
